package javafx3d01;

import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.PerspectiveCamera;
import javafx.scene.PointLight;
import javafx.scene.Scene;
import javafx.scene.SubScene;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.*;
import javafx.stage.Stage;
import jfxtras.labs.scene.control.window.CloseIcon;
import jfxtras.labs.scene.control.window.MinimizeIcon;
import jfxtras.labs.scene.control.window.RotateIcon;
import jfxtras.labs.scene.control.window.Window;

public class JavaFX3D01 extends Application {

    @Override
    public void start(Stage stage) {

        // create scalable root pane
        ScalableContentPane scalableContent = new ScalableContentPane();
        Pane canvas = scalableContent.getContentPane();

        // create a few windows with 3D content
        for (int i = 0; i < 9; i++) {
            
            boolean useSubScene = i % 2 ==0;
            
            Window w = createAndAdd3DWindow(canvas, useSubScene);
            
            w.setLayoutX((i % 3) * (w.getPrefWidth()+ 30));
            w.setLayoutY((i / 3) * (w.getPrefHeight()+ 30));
        }

        // the usual scene/stage setup
        final Scene scene = new Scene(scalableContent, 800, 800, true);
        scene.setFill(Color.rgb(10, 10, 40));
        scene.setCamera(new PerspectiveCamera(false));
        stage.setScene(scene);
        stage.show();
        
    }

    public static void main(String[] args) {
        launch(args);
    }

    private Node create3DContent(final Window w, boolean subscene) {

        // add geometries to a group
        // see: http://fxexperience.com/2013/02/javafx-3d-early-access-available/

        PhongMaterial material = new PhongMaterial();

        material.setDiffuseColor(Color.rgb(50, 60, 100));
        material.setSpecularColor(Color.rgb(30, 30, 30));

        Shape3D[] meshView = new Shape3D[]{
            new Box(200, 200, 200),
            new Sphere(100),
            new Cylinder(100, 200),};

        for (int i = 0; i != 3; ++i) {
            meshView[i].setMaterial(material);
            meshView[i].setTranslateX((i + 1) * 220);
            meshView[i].setTranslateY(200);
            meshView[i].setTranslateZ(-100);
            meshView[i].setDrawMode(DrawMode.FILL);
            meshView[i].setCullFace(CullFace.BACK);

            final int index = i;

            // does only work without pointlight. very strange?
            meshView[i].setOnMouseClicked((MouseEvent event) -> {
                w.setTitle("Geometry " + (index  + 1) + " Selected");
            });

        }

        Group root3D = new Group(meshView);

        // light setup
        PointLight pointLight = new PointLight(Color.WHITE);
        pointLight.setTranslateX(800);
        pointLight.setTranslateY(-100);
        pointLight.setTranslateZ(-1000);

        root3D.getChildren().add(pointLight);

        Node result = root3D;

        if (subscene) {
            SubScene scene3D = new SubScene(
                    root3D,
                    1000,
                    600, true, true);
            scene3D.setCamera(new PerspectiveCamera(false));
            result = scene3D;
        }

        return result;
    }

    private Window createAndAdd3DWindow(Pane canvas, boolean subscene) {

        // create a window control
        Window w = new Window();

        // set some nice style
        w.setStyle(
                "-fx-background-color: rgba(120,140,255,0.2);"
                + "-fx-border-color: rgba(120,140,255,0.42);"
                + "-fx-border-width: 2;");
        
        w.setTitle("3D Window");

        // window content shall be scalable
        ScalableContentPane wContent = new ScalableContentPane();
        Pane wContentPane = wContent.getContentPane();

        // we have to define a clip
        // otherwise layout does not work properly
        Rectangle clip = new Rectangle();
        clip.widthProperty().bind(wContent.widthProperty());
        clip.heightProperty().bind(wContent.heightProperty());
        wContent.setClip(clip);

        // create 3d content
        Node root3D = create3DContent(w, subscene);

        // set window content
        wContentPane.getChildren().add(root3D);
        w.setContentPane(wContent);

        // add the wondow to the canvas
        canvas.getChildren().add(w);

        // define the window size
        w.setPrefSize(400, 400);

        // add some icons
        w.getLeftIcons().add(new CloseIcon(w));
        w.getLeftIcons().add(new MinimizeIcon(w));

        // question: why is this icon not visible?
        w.getRightIcons().add(new RotateIcon(w));

        return w;
    }
}